@@ -17,7 +17,7 @@
 #include "afs/param.h"
 
 RCSID
-    ("$Header: /cvs/openafs/src/afs/afs_init.c,v 1.28.2.1 2004/12/07 06:12:11 shadow Exp $");
+    ("$Header: /cvs/openafs/src/afs/afs_init.c,v 1.37.4.3 2006/07/31 21:27:38 shadow Exp $");
 
 #include "afs/stds.h"
 #include "afs/sysincludes.h"	/* Standard vendor system headers */
@@ -27,7 +27,7 @@ RCSID
 /* Exported variables */
 struct osi_dev cacheDev;	/*Cache device */
 afs_int32 cacheInfoModTime;	/*Last time cache info modified */
-#if defined(AFS_OSF_ENV) || defined(AFS_DEC_ENV) || defined(AFS_DARWIN_ENV) || defined(AFS_FBSD_ENV)
+#if defined(AFS_OSF_ENV) || defined(AFS_DARWIN_ENV) || defined(AFS_FBSD_ENV)
 struct mount *afs_cacheVfsp = 0;
 #elif defined(AFS_LINUX20_ENV)
 struct super_block *afs_cacheSBp = 0;
@@ -39,6 +39,7 @@ char *afs_sysname = 0;		/* So that superuser may change the
 				 * local value of @sys */
 char *afs_sysnamelist[MAXNUMSYSNAMES];	/* For support of a list of sysname */
 int afs_sysnamecount = 0;
+int afs_sysnamegen = 0;
 struct volume *Initialafs_freeVolList;
 int afs_memvolumes = 0;
 #if defined(AFS_XBSD_ENV)
@@ -111,26 +112,6 @@ afs_CacheInit(afs_int32 astatSize, afs_int32 afiles, afs_int32 ablocks,
     RWLOCK_INIT(&afs_xaxs, "afs_xaxs");
     osi_dnlc_init();
 
-
-#if	defined(AFS_AIX32_ENV) || defined(AFS_HPUX_ENV)
-    {
-	afs_int32 preallocs;
-
-	/*
-	 * We want to also reserve space for the gnode struct which is associated
-	 * with each vnode (vcache) one; we want to use the pinned pool for them   
-	 * since they're referenced at interrupt level.
-	 */
-	if (afs_stats_cmperf.SmallBlocksAlloced + astatSize < 3600)
-	    preallocs = astatSize;
-	else {
-	    preallocs = 3600 - afs_stats_cmperf.SmallBlocksAlloced;
-	    if (preallocs <= 0)
-		preallocs = 10;
-	}
-	osi_AllocMoreSSpace(preallocs);
-    }
-#endif
     /* 
      * create volume list structure 
      */
@@ -237,25 +218,21 @@ LookupInodeByPath(char *filename, ino_t * inode, struct vnode **fvpp)
 
 #ifdef AFS_LINUX22_ENV
     struct dentry *dp;
-    code = gop_lookupname(filename, AFS_UIOSYS, 0, NULL, &dp);
+    code = gop_lookupname(filename, AFS_UIOSYS, 0, &dp);
     if (code)
 	return code;
     *inode = dp->d_inode->i_ino;
     dput(dp);
 #else
     struct vnode *filevp;
-    code = gop_lookupname(filename, AFS_UIOSYS, 0, NULL, &filevp);
+    code = gop_lookupname(filename, AFS_UIOSYS, 0, &filevp);
     if (code)
 	return code;
     *inode = afs_vnodeToInumber(filevp);
     if (fvpp)
 	*fvpp = filevp;
     else {
-#if defined(AFS_DEC_ENV)
-	grele(filevp);
-#else
 	AFS_RELE(filevp);
-#endif
     }
 #endif /* AFS_LINUX22_ENV */
 
@@ -364,7 +341,7 @@ afs_InitCacheInfo(register char *afile)
     if (code)
 	return code;
 #else
-    code = gop_lookupname(afile, AFS_UIOSYS, 0, NULL, &filevp);
+    code = gop_lookupname(afile, AFS_UIOSYS, 0, &filevp);
     if (code || !filevp)
 	return ENOENT;
     {
@@ -376,6 +353,8 @@ afs_InitCacheInfo(register char *afile)
 	struct statvfs st;
 #elif defined(AFS_DUX40_ENV)
 	struct nstatfs st;
+#elif defined(AFS_DARWIN80_ENV)
+	struct vfsstatfs st;
 #else
 	struct statfs st;
 #endif /* SUN56 */
@@ -405,6 +384,9 @@ afs_InitCacheInfo(register char *afile)
 	    VFS_STATFS(filevp->v_vfsp, &st);
 	    TO_KERNEL_SPACE();
 	}
+#elif defined(AFS_DARWIN80_ENV)
+        afs_cacheVfsp = vnode_mount(filevp);
+	if (afs_cacheVfsp && ((st = *(vfs_statfs(afs_cacheVfsp))),1))
 #elif defined(AFS_DARWIN_ENV)
 	if (!VFS_STATFS(filevp->v_mount, &st, current_proc()))
 #elif defined(AFS_FBSD50_ENV)
@@ -437,7 +419,9 @@ afs_InitCacheInfo(register char *afile)
 #endif
     cacheInode = afs_vnodeToInumber(filevp);
     cacheDev.dev = afs_vnodeToDev(filevp);
+#ifndef AFS_DARWIN80_ENV
     afs_cacheVfsp = filevp->v_vfsp;
+#endif
 #endif /* AFS_LINUX20_ENV */
     AFS_RELE(filevp);
 #endif /* AFS_LINUX22_ENV */
@@ -492,9 +476,7 @@ afs_ResourceInit(int preallocs)
     RWLOCK_INIT(&afs_xinterface, "afs_xinterface");
     LOCK_INIT(&afs_puttofileLock, "afs_puttofileLock");
 #ifndef AFS_FBSD_ENV
-#ifndef	AFS_AIX32_ENV
     LOCK_INIT(&osi_fsplock, "osi_fsplock");
-#endif
     LOCK_INIT(&osi_flplock, "osi_flplock");
 #endif
     RWLOCK_INIT(&afs_xconn, "afs_xconn");
@@ -511,15 +493,7 @@ afs_ResourceInit(int preallocs)
 	afs_sysname = afs_sysnamelist[0];
 	strcpy(afs_sysname, SYS_NAME);
 	afs_sysnamecount = 1;
-#if	defined(AFS_AIX32_ENV) || defined(AFS_HPUX_ENV)
-	{
-
-	    if ((preallocs > 256) && (preallocs < 3600))
-		afs_preallocs = preallocs;
-	    osi_AllocMoreSSpace(afs_preallocs);
-	    osi_AllocMoreMSpace(100);
-	}
-#endif
+	afs_sysnamegen++;
     }
 
     secobj = rxnull_NewServerSecurityObject();
@@ -691,7 +665,7 @@ shutdown_cache(void)
 void
 shutdown_vnodeops(void)
 {
-#if !defined(AFS_SGI_ENV) && !defined(AFS_SUN_ENV) && !defined(AFS_SUN5_ENV)
+#if !defined(AFS_SGI_ENV) && !defined(AFS_SUN5_ENV)
     struct buf *afs_bread_freebp = 0;
 #endif
 
@@ -704,7 +678,7 @@ shutdown_vnodeops(void)
 #ifndef AFS_LINUX20_ENV
 	afs_rd_stash_i = 0;
 #endif
-#if !defined(AFS_SGI_ENV) && !defined(AFS_SUN_ENV) && !defined(AFS_SUN5_ENV)
+#if !defined(AFS_SGI_ENV) && !defined(AFS_SUN5_ENV)
 	afs_bread_freebp = 0;
 #endif
 	shutdown_mariner();
