@@ -19,13 +19,16 @@
 #endif
 
 RCSID
-    ("$Header: /cvs/openafs/src/rxkad/rxkad_client.c,v 1.18 2004/04/19 05:43:58 kolya Exp $");
+    ("$Header: /cvs/openafs/src/rxkad/rxkad_client.c,v 1.22.2.1 2006/06/01 15:38:59 shadow Exp $");
 
 #ifdef KERNEL
 #include "afs/stds.h"
 #ifndef UKERNEL
 #include "h/types.h"
 #include "h/time.h"
+#if defined(AFS_AIX_ENV) || defined(AFS_AUX_ENV) || defined(AFS_SUN5_ENV) 
+#include "h/systm.h"
+#endif
 #ifdef AFS_LINUX20_ENV
 #include "h/socket.h"
 #endif
@@ -63,6 +66,7 @@ RCSID
 #endif /* AFS_PTHREAD_ENV */
 #endif /* KERNEL */
 
+#include <des/stats.h>
 #include "private_data.h"
 #define XPRT_RXKAD_CLIENT
 
@@ -101,8 +105,8 @@ static struct rx_securityOps rxkad_client_ops = {
  */
 #include <assert.h>
 pthread_mutex_t rxkad_client_uid_mutex;
-#define LOCK_CUID assert(pthread_mutex_lock(&rxkad_client_uid_mutex)==0);
-#define UNLOCK_CUID assert(pthread_mutex_unlock(&rxkad_client_uid_mutex)==0);
+#define LOCK_CUID assert(pthread_mutex_lock(&rxkad_client_uid_mutex)==0)
+#define UNLOCK_CUID assert(pthread_mutex_unlock(&rxkad_client_uid_mutex)==0)
 #else
 #define LOCK_CUID
 #define UNLOCK_CUID
@@ -119,7 +123,8 @@ rxkad_AllocCID(struct rx_securityClass *aobj, struct rx_connection *aconn)
     struct rxkad_cidgen tgen;
     static afs_int32 counter = 0;	/* not used anymore */
 
-    LOCK_CUID if (Cuid[0] == 0) {
+    LOCK_CUID;
+    if (Cuid[0] == 0) {
 	afs_uint32 xor[2];
 	tgen.ipAddr = rxi_getaddr();	/* comes back in net order */
 	clock_GetTime(&tgen.time);	/* changes time1 and time2 */
@@ -154,12 +159,14 @@ rxkad_AllocCID(struct rx_securityClass *aobj, struct rx_connection *aconn)
     }
 
     if (!aconn) {
-	UNLOCK_CUID return 0;
+	UNLOCK_CUID;
+	return 0;
     }
     aconn->epoch = Cuid[0];
     aconn->cid = Cuid[1];
     Cuid[1] += 1 << RX_CIDSHIFT;
-    UNLOCK_CUID return 0;
+    UNLOCK_CUID;
+    return 0;
 }
 
 /* Allocate a new client security object.  Called with the encryption level,
@@ -174,7 +181,7 @@ rxkad_NewClientSecurityObject(rxkad_level level,
     struct rx_securityClass *tsc;
     struct rxkad_cprivate *tcp;
     int code;
-    int size;
+    int size, psize;
 
     size = sizeof(struct rx_securityClass);
     tsc = (struct rx_securityClass *)rxi_Alloc(size);
@@ -182,15 +189,15 @@ rxkad_NewClientSecurityObject(rxkad_level level,
     tsc->refCount = 1;		/* caller gets one for free */
     tsc->ops = &rxkad_client_ops;
 
-    size = sizeof(struct rxkad_cprivate);
-    tcp = (struct rxkad_cprivate *)rxi_Alloc(size);
-    memset((void *)tcp, 0, size);
+    psize = PDATA_SIZE(ticketLen);
+    tcp = (struct rxkad_cprivate *)rxi_Alloc(psize);
+    memset((void *)tcp, 0, psize);
     tsc->privateData = (char *)tcp;
     tcp->type |= rxkad_client;
     tcp->level = level;
     code = fc_keysched(sessionkey, tcp->keysched);
     if (code) {
-	rxi_Free(tcp, sizeof(struct rxkad_cprivate));
+	rxi_Free(tcp, psize);
 	rxi_Free(tsc, sizeof(struct rx_securityClass));
 	return 0;		/* bad key */
     }
@@ -198,14 +205,14 @@ rxkad_NewClientSecurityObject(rxkad_level level,
     tcp->kvno = kvno;		/* key version number */
     tcp->ticketLen = ticketLen;	/* length of ticket */
     if (tcp->ticketLen > MAXKTCTICKETLEN) {
-	rxi_Free(tcp, sizeof(struct rxkad_cprivate));
+	rxi_Free(tcp, psize);
 	rxi_Free(tsc, sizeof(struct rx_securityClass));
 	return 0;		/* bad key */
     }
     memcpy(tcp->ticket, ticket, ticketLen);
 
-    LOCK_RXKAD_STATS rxkad_stats_clientObjects++;
-    UNLOCK_RXKAD_STATS return tsc;
+    INC_RXKAD_STATS(clientObjects);
+    return tsc;
 }
 
 /* client: respond to a challenge packet */
@@ -250,8 +257,8 @@ rxkad_GetResponse(struct rx_securityClass *aobj, struct rx_connection *aconn,
 
     if (level > tcp->level)
 	return RXKADLEVELFAIL;
-    LOCK_RXKAD_STATS rxkad_stats.challenges[rxkad_LevelIndex(tcp->level)]++;
-    UNLOCK_RXKAD_STATS if (v2) {
+    INC_RXKAD_STATS(challenges[rxkad_LevelIndex(tcp->level)]);
+    if (v2) {
 	int i;
 	afs_uint32 xor[2];
 	memset((void *)&r_v2, 0, sizeof(r_v2));
@@ -307,6 +314,8 @@ rxkad_GetResponse(struct rx_securityClass *aobj, struct rx_connection *aconn,
 void
 rxkad_ResetState(void)
 {
-    LOCK_CUID Cuid[0] = 0;
+    LOCK_CUID;
+    Cuid[0] = 0;
     rxkad_EpochWasSet = 0;
-UNLOCK_CUID}
+    UNLOCK_CUID;
+}
